/**
 * REST API Tests
 */

function json(verb, url) {
  return request(app)[verb](url)
  .set('Content-Type', 'application/json')
  .set('Accept', 'application/json')
  .expect('Content-Type', /json/);
}

describe('REST', function() {
  /**
   * Expected Input Tests
   */

  describe('Expected Usage', function() {

    describe('GET /api/cars', function() {
      it('should return a list of all cars', function(done) {
        json('get', '/api/cars')
          .expect(200)
          .end(function(err, res) {
            assert(Array.isArray(res.body));
            assert.equal(res.body.length, testData.cars.length);

            done();
          });
      });
    });

    describe('POST /api/cars', function() {
      it('should create a new car', function(done) {
        json('post', '/api/cars')
          .send({
            "vin": "ebaddaa5-35bb-4b33-a388-87203acb6478",
            "year": "2013",
            "make": "Dodge",
            "model": "Taurus",
            "image": "/images/car/car_0.jpg",
            "carClass": "suv",
            "color": "white"
          })
          .expect(200)
          .end(function(err, res) {
            assert(typeof res.body === 'object');
            assert(res.body.id, 'must have an id');
            done();
          });
      });
    });

    describe('PUT /api/cars/:id', function() {
      it('should update a car with the given id', function(done) {
        json('get', '/api/cars')
          .expect(200, function(err, res) {
            var cars = res.body;
            var car = cars[0];

            json('put', '/api/cars/' + car.id)
              .send({
                year: 2000,
                color: 'red'
              })
              .expect(200, function(err, res) {
                var updatedCar = res.body;
                assert(updatedCar);
                assert(updatedCar.id);
                assert.equal(updatedCar.id, car.id);
                assert.equal(updatedCar.year, 2000);
                json('get', '/api/cars/' + car.id)
                  .expect(200, function(err, res) {
                    var foundCar = res.body;
                    assert.equal(foundCar.id, car.id);
                    assert.equal(foundCar.year, 2000);
                    assert.equal(foundCar.color, 'red');
                    done();
                  });
              });
          });
      });
    });

    describe('GET /api/locations', function() {
      it('should return a list of locations', function(done) {
        json('get', '/api/locations')
          .expect(200, function(err, res) {
            var locations = res.body;
            assert(Array.isArray(locations));
            assert.equal(locations.length, testData.locations.length);
            done();
          });
      });
    });

    describe('GET /api/locations/nearby', function() {
      it('should return a list of locations near given point', function(done) {
        json('get', '/api/locations/nearby?here[lat]=37.7883415&here[lng]=-122.4209035')
          .expect(200, function(err, res) {
            var locations = res.body;
            assert(Array.isArray(locations));
            assert.equal(locations[0].name, 'City Rent-a-Car');
            assert.equal(locations.length, 10);
            locations.forEach(function(l) {
              assert(l.geo);
              assert.equal(typeof l.geo.lat, 'number');
              assert.equal(typeof l.geo.lng, 'number');
            });
            assert.equal(locations[locations.length - 1].city, 'San Francisco');
            done();
          });
      });
    });

    describe('GET /api/locations/:id/inventory', function() {
      it('should return a list of inventory for the given location id', function(done) {
        json('get', '/api/locations/5/inventory')
          .expect(200, function(err, res) {
            var inventory = res.body;
            assert.equal(inventory.length, 87);
            inventory.forEach(function(inv) {
              assert.equal(typeof inv.total, 'number');
              assert.equal(typeof inv.available, 'number');
            });
            done();
          });
      });
    });

    // describe('GET /api/customers', function(){
    //   it('should return a 401 when not logged in as an admin', function(done) {
    //     json('get', '/api/customers').expect(401, done);
    //   });
    //
    //   it('should return all users when logged in as an admin', function(done) {
    //
    //   });
    // });

    describe('/api/customers', function() {
      var credentials = { email: 'a-@example.com', password: 'a-password' };
      var customer;
      var token;
      it('should create new customer on POST', function(done) {
        json('post', '/api/customers')
          .send(credentials)
          .expect(200, function(err, res) {
            if (err) return done(err);
            customer = res.body;
            assert.equal(customer.email, credentials.email);
            done();
          });
      });

      it('should login existing customer on POST /api/customers/login', function(done) {
        json('post', '/api/customers/login')
          .send(credentials)
          .expect(200, function(err, res) {
            if (err) return done(err);
            token = res.body;
            assert.equal(token.userId, customer.id);
            done();
          });
      });

      it('should allow GET /api/customers/{my-id}', function(done) {
        json('get', '/api/customers/' + customer.id)
          .set('Authorization', token.id)
          .expect(200, function(err, res) {
            if (err) return done(err);
            assert.equal(customer.email, res.body.email);
            done();
          });
      });

      it('should not allow GET /api/customers/{another-id}', function(done) {
        json('get', '/api/customers/' + (customer.id + 1000))
          .set('Authorization', token.id)
          .expect(401, function(err) {
            done(err);
          });
      });

      it('should logout existing customer on POST /api/customers/logout', function(done) {
        json('post', '/api/customers/logout')
          .set('Authorization', token.id)
          .send({})
          .expect(204, done);
      });
    });
  });

  describe('Unexpected Usage', function(){
     describe('POST /api/cars/:id', function(){
       it('should not crash the server when posting a bad id', function(done) {
         json('post', '/api/cars/foobar').send({}).expect(404, done);
       });
     });
   });

});
